function [corrScore, boundingBox] = corrMatching(frameImg, templateImg,numrot, threshC)
% -------------------------------------------------------------------------
% Function corrMatching: Template Matching using Correlation Coefficients
% Inputs: 
%           frameImg = gray or color frame image
%           templateImg = gray or color template image
%           threshC = threshold of rejecting detected region (default = .75)
%                     e.g. if the detected region has a corrCoef>threshC
%                     then the algorithm accepts it as a detection,
%                     otherwise rejects it as a false alarm.
%
%           numrot - the number of angles of the logo could be in the
%           main picture
% Output: 
%           corrScore = 2D matrix of correlation coefficients
%           boundingBox = [upperLeftPixel.y upperLeftPixel.x height width]
%
% -------------------------------------------------------------------------
% By Yue Wu (Rex) 
% Department of Electrical and Computer Engineering
% Tufts University
% Medford, MA
% 08/30/2010
%
%Improved by Lihi Wishnitzer and Gil Matzov
%05/03/2012
% -------------------------------------------------------------------------

%% 1. initialization
if size(frameImg,3) ~=1
    frameGray = rgb2gray(frameImg);
else
    frameGray = frameImg;
end
frameGray = double(frameGray);

if size(templateImg,3) ~=1
    templateGray = rgb2gray(templateImg);
else
    templateGray = templateImg;
end
templateGray = double(templateGray);
[templateHeight,templateWidth] = size(templateGray);
templateGrayOrg = templateGray;

theAngele = 0;
if  0 ~= numrot
angele2Rotate = 360/numrot;
else
    angele2Rotate = 0;
end
 boundingBox(1,:) = [0 0 0 0];
%% 2. correlation calculation
for i=0:numrot
    
frameMean = conv2(frameGray,ones(size(templateGray))./numel(templateGray),'same');
templateMean = mean(templateGray(:));
corrPartI = conv2(frameGray,rot90(templateGray-templateMean,2),'same')./numel(templateGray);
corrPartII = frameMean.*sum(templateGray(:)-templateMean);
stdFrame = sqrt(conv2(frameGray.^2,ones(size(templateGray))./numel(templateGray),'same')-frameMean.^2);
stdTemplate = std(templateGray(:));
corrScore = (corrPartI-corrPartII)./(stdFrame.*stdTemplate);

%% 3. finding most likely region
[maxVal,maxIdx] = max(corrScore(:));
[maxR, maxC] = ind2sub([size(corrScore,1),size(corrScore,2)],maxIdx);

%% 4. hypothesis test
if ~exist('threshC','var')
    threshC = .46;
end
if maxVal>=threshC
    boundingBox(1,:) = [max(1,maxR-round(templateHeight/2)), max(1,maxC-round(templateWidth/2)), templateHeight, templateWidth];
    break;
else
    theAngele = theAngele + angele2Rotate;
    templateGray = imrotate(templateGrayOrg,theAngele,'bicubic','loose');
    [templateHeight,templateWidth] = size(templateGray);
end
end
